/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.operator;

import java.io.*;
import java.lang.reflect.Field;

import de.unihalle.informatik.Alida.annotations.Parameter;

/**
 *  This class describes the parameters of an operator.
 *  <p>
 *  A <code>ALDOpParameterDescriptor</code> has a direction (IN, OUT, INOUT) and 
 *  a boolean indicating if this parameter is supplemental.
 * 
 * @author posch
 */
public class ALDOpParameterDescriptor extends ALDParameterDescriptor
	implements Cloneable {

	protected final Parameter.Direction direction;

	protected final Boolean supplemental;

	/** Construct a descriptor 
	 *
	 * @param	name					Name of parameter.
	 * @param	direction		Direction of parameter: IN/OUT/INOUT.
	 * @param	supplemental Supplemental parameter flag.
	 * @param	cl				Java class. 
	 * @param	explanation 	Explanatory string.
	 * @param label				Label for parameter.
	 * @param	required			Is this argument required for the operator?
	 * @param	defaultValue 			Default value.
	 * @param	field 				The field for this member.
	 * @param guiOrder			Position in GUI of this member.
	 * @param mode					Mode for handling the parameter.
	 */
	public ALDOpParameterDescriptor(String name, Parameter.Direction direction, 
			Boolean supplemental, Class<?> cl, String explanation,
			String label, boolean required, Object defaultValue, Field field,
			int guiOrder, Parameter.ExpertMode mode) {
		super(name, cl, explanation, label, 
				required, defaultValue, field,guiOrder,mode);
		this.direction = direction;
		this.supplemental = supplemental;
	}
	
	/**
	 */
	public ALDOpParameterDescriptor copy( Field field) {
		ALDOpParameterDescriptor obj = new ALDOpParameterDescriptor( this.name,
				this.direction, this.supplemental, this.myclass, this.explanation, 
				this.label, this.required, this.defaultValue, field,this.dataIOOrder,
				this.handlingMode);
		return obj;
	}



	/**
	 * Clone an instance
	 */
	@Override
	public ALDOpParameterDescriptor clone() {
		ALDOpParameterDescriptor obj = new ALDOpParameterDescriptor( this.name,
				this.direction, this.supplemental, this.myclass, this.explanation, 
				this.label, this.required, this.defaultValue, this.field,this.dataIOOrder,
				this.handlingMode);
		return obj;
	}

	/** 
	 * Print this descriptor to outfile
	 */
	@Override
	public void print( PrintStream outfile) {
		print( outfile, null);
	}

	/** 
	 * Print this descriptor to outfile.
	 * <p>
	 * If <code>op</code> is non null then try to get the value of the parameter and print it also.
	 */
	public void print( PrintStream outfile, ALDOperator op) {
		Object value = null;
		if ( op != null ) {
			try {
				value = op.getParameter( this.name);
			} catch (Exception ex) {
				// nothing happens here...
			}
		}

		outfile.println( "  Parameter <" + this.name +  "> " +
				(op != null ? " = " +  value + " ": "" ) 
				+	"(" + this.direction  
				+ (this.supplemental.booleanValue() ? " supplemental)" : ")") 
				+	(this.required ? " required" : "") +
				", type: " + this.myclass.getSimpleName() +
				", label: " + this.label +
				", (" + this.explanation + ")");
	}

	/**
	 * Returns parameter direction.
	 */
	public Parameter.Direction getDirection() {
		return this.direction;
	}

	/**
	 * Returns if parameter is supplemental.
	 */
	public Boolean getSupplemental() {
		return this.supplemental;
	}
}
